package org.hmn.jrsd.demo;

import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/*
 * 
 * Cree par Harry Karadimas a 9 sept. 08
 */
public class H2Utils
{
  
  public static boolean tableExists(Connection cxn, String name)
  throws SQLException
  {    
    DatabaseMetaData md = cxn.getMetaData();
    ResultSet rs = md.getTables(null, null, name, null);
    int n = 0;
    while (rs.next()) {
      rs.getString(1);
      n++;
    }
    return n > 0;
  }
  
  public static boolean sequenceExists(Connection cxn, String name)
  throws SQLException
  {
    String sql = 
      "SELECT COUNT(*)"+
      " FROM INFORMATION_SCHEMA.SEQUENCES"+
      " WHERE SEQUENCE_NAME=?";
    PreparedStatement st = null;
    try {
      st = cxn.prepareStatement(sql);
      st.setString(1, name);
      ResultSet rs = st.executeQuery();
      int n = 0;
      while (rs.next()) {
        n = rs.getInt(1);
      }
      return n > 0;
    }
    finally {
      if (st != null) st.close();
    }
  }
  
  public static void ddl(Connection cxn, String sql)
  throws SQLException
  {
    Statement st = null;
    try {      
      System.out.println(sql);
      st = cxn.createStatement();
      st.executeUpdate(sql);
    }
    finally {
      if (st != null) st.close();
    }    
  }
  
  public static void ddlResource(Connection cxn, String resource)
  throws IOException, SQLException
  {
    ClassLoader cldr = new H2Utils().getClass().getClassLoader();
    InputStream is = cldr.getResourceAsStream(resource);
    if (is == null) {
      throw new FileNotFoundException("Resource not found : "+resource);
    }
    int c = 0;
    StringBuffer sb = new StringBuffer();
    while ((c = is.read()) >= 0) sb.append((char)c);
    is.close();
    ddl(cxn, stripTrailingExtraChars(sb.toString()));
  }
  
  public static String stripTrailingExtraChars(String sql)
  {
    String s = sql;
    for (;;) {
      if (s.length() == 0) return "";
      char c = s.charAt(s.length()-1);
      if (c == ' ' | c == '\t' | c == ';' | c == '\n' | c == '\r') {
        s = s.substring(0, s.length() - 1);
      }
      else return s;
    }
  }
  
}
