package org.hmn.jrsd.demo;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.sql.DataSource;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.hmn.jrsd.demo.orm.CDS;
import org.hmn.jrsd.rt.JrsdRtUtils;
import org.hmn.jrsd.rt.SormUtils;

import simpleorm.dataset.SDataSet;
import simpleorm.dataset.SFieldBoolean;
import simpleorm.dataset.SFieldMeta;
import simpleorm.dataset.SFieldScalar;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;
import simpleorm.sessionjdbc.SSessionJdbc;

/*
 * 
 * Cree par Harry Karadimas a 9 sept. 08
 */
/**
 * A simple table model that uses a list of SRecordInstance directly.
 * Not very generic, for this demo only.
 */
public class JrsdRecordTableModel<RI extends SRecordInstance>
implements TableModel
{
  List<RI> recs;
  SRecordMeta<RI> meta;
  List<TableModelListener> listeners = new ArrayList<TableModelListener>();
  private static HashMap<Integer, Class<?>> typeMap;
  DataSource ds;
  HashMap<SFieldMeta, TableFieldHandler> fieldHandlers = 
    new HashMap<SFieldMeta, TableFieldHandler>();
  
  {
    typeMap = new HashMap<Integer, Class<?>>();
    tmd(Types.BIGINT, BigDecimal.class);
    tmd(Types.BOOLEAN, Boolean.class);
    tmd(Types.INTEGER, Long.class);
    tmd(Types.REAL, Double.class);
    tmd(Types.DATE, Timestamp.class);
    tmd(Types.TIME, Timestamp.class);
    tmd(Types.TIMESTAMP, Timestamp.class);
    tmd(Types.CHAR, String.class);
    tmd(Types.VARCHAR, String.class);
    tmd(Types.FLOAT, Double.class);
  }
  
  private static void tmd(int t, Class<?> cl) { typeMap.put(t, cl); }
  
  public JrsdRecordTableModel(DataSource ds, SRecordMeta<RI> meta, List<RI> recs)
  {
    this.ds = ds;
    this.recs = recs;
    this.meta = meta;
  }
  
  public void addTableModelListener(TableModelListener l) {
    listeners.add(l);
  }

  public Class<?> getColumnClass(int columnIndex) {
    SFieldScalar fsc = meta.getAllScalarFields()[columnIndex];
    TableFieldHandler fh = fieldHandlers.get(fsc);
    if (fh != null) return fh.getColumnClass(columnIndex);    
    //SFieldBoolean is a special case, as it is represented by a CHAR typically
    if (fsc instanceof SFieldBoolean) return Boolean.class;
    int sqlType = fsc.javaSqlType();    
    return typeMap.get(sqlType);
  }

  public int getColumnCount() {
    return meta.getAllScalarFields().length;
  }

  public String getColumnName(int columnIndex) 
  {
    SFieldMeta fm = meta.getAllScalarFields()[columnIndex];
    TableFieldHandler fh = fieldHandlers.get(fm);    
    if (fh != null) return fh.getColumnName(columnIndex);    
    return meta.getAllScalarFields()[columnIndex].getColumnName();
  }

  public int getRowCount() {
    return recs.size();
  }

  public Object getValueAt(int rowIndex, int columnIndex) 
  {
    SFieldMeta fm = meta.getAllScalarFields()[columnIndex];
    TableFieldHandler fh = fieldHandlers.get(fm);    
    if (fh != null) return fh.getValueAt(rowIndex, columnIndex);    
    RI rec = recs.get(rowIndex);
    return rec.getObject(meta.getAllScalarFields()[columnIndex]);    
  }

  public boolean isCellEditable(int rowIndex, int columnIndex) 
  {    
    SFieldMeta fm = meta.getAllScalarFields()[columnIndex];
    if (fm.equals(CDS.LENDED_F)) return false;
    TableFieldHandler fh = fieldHandlers.get(fm);
    if (fh != null) return fh.isCellEditable(rowIndex, columnIndex);    
    if (SormUtils.isPartOfPrimaryKey(fm)) return false;
    return true;
  }

  public void removeTableModelListener(TableModelListener l) {
    listeners.remove(l);
  }

  public void setValueAt(Object value, int rowIndex, int columnIndex) 
  {
    RI rec = recs.get(rowIndex);
    rec.setObject(meta.getAllScalarFields()[columnIndex], value);
    SDataSet dats = rec.getDataSet();
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    if (session == null) session = SSessionJdbc.open(ds, "setValueAt");
    try {
      //reattach data set
      session.begin(dats);
      //commit, flushes the values that changed, verifies that optimistic locking worked
      session.commitAndDetachDataSet();
    }
    finally {
      session.close();
    }
  }

  public void setRecords(List<RI> newRecs)
  {
    recs = newRecs;
    TableModelEvent evt = new TableModelEvent(this);
    for (TableModelListener listener:listeners) {
      listener.tableChanged(evt);
    }
  }
  
  public List<RI> getRecords() { return recs; }
  
  public RI getRecord(int index) { return recs.get(index); }
  
  public void add(TableFieldHandler fh)
  {
    fieldHandlers.put(fh.getSFieldScalar(), fh);
  }
}
