package org.hmn.jrsd.demo;

import java.io.IOException;
import java.sql.Connection;
import java.util.HashMap;

import javax.swing.JOptionPane;

import org.hmn.jrsd.demo.orm.CDS;
import org.hmn.jrsd.demo.orm.FRIENDS;

import simpleorm.dataset.SGeneratorMode;
import simpleorm.sessionjdbc.SSessionJdbc;
import simpleorm.utils.SLog;
import simpleorm.utils.SLogLog4jBridge;
import simpleorm.utils.SLogLog4jBridge.Configuration;

/*
 * 
 * Cree par Harry Karadimas a 8 sept. 08
 */
/**
 * A simple application to demonstrate the use of jrsd + simpleorm. You have
 * CDS and FRIENDS; you manage the CDS that you lend to you FRIENDS. Swing
 * is used for the GUI, database sessions are opened as needed, and closed
 * as quickly as possible. Normally Swing ensures that all connections occur
 * in the same thread.
 */
public class LendingCds
{
  //names of sql files for table creation
  private static final String SQL_TABLE_CDS = 
    "sql/table-CDS_v1_1703253007.sql";
  private static final String SQL_TABLE_FRIENDS = 
    "sql/table-FRIENDS_v1_345704086.sql";                                                        
  
  LendingCdsProperties props;
  H2DataSource ds;
  LendingCdsFrame frame;
  //HashMap<Integer, Friend> friendsById = new HashMap<Integer, Friend>();
  SLogLog4jBridge slg;
  Configuration slgConf;
  
  public void init(String initPropsFilePath)
  throws Exception
  {
    SLogLog4jBridge.useSLogLog4jBridge();
    SLogLog4jBridge.getDefaultConfiguration()
        .setLevel(SLogLog4jBridge.LN_CONNECTIONS, "info");
    
    //we register the H2 driver
    new H2Driver().registerDriver();
    //init generators used
    //CDS.cCDS_ID.setGeneratorMode(SGeneratorMode.SSEQUENCE, "CDS_ID_SEQ");
    //now load app properties for further init
    props = new LendingCdsProperties(initPropsFilePath);
    String msg = props.checkProperties();
    if (msg != null) {
      JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
      //stop immediately, so throwing a RuntimeException is OK
      throw new RuntimeException(msg);
    }
    String url = props.getJdbcUrl();
    String user = props.getJdbcUser();
    String password = props.getJdbcPassword();
    ds = new H2DataSource();
    ds.setUrl(url);
    ds.setUser(user);
    ds.setPassword(password);
    setUpDatabase(ds.getConnection());
    frame = new LendingCdsFrame(this);
    frame.setDatasource(ds);
    frame.openSessionForSwing();
  }
  
  /**
   * Set up database, recreate if necessary
   * @param cxn
   * @throws Exception
   */
  private void setUpDatabase(Connection cxn)
  throws Exception
  {    
    if (!H2Utils.sequenceExists(cxn, "CDS_ID_SEQ")) {
      H2Utils.ddl(cxn, "CREATE SEQUENCE CDS_ID_SEQ");
    }
    if (!H2Utils.sequenceExists(cxn, "FRIENDS_ID_SEQ")) {
      H2Utils.ddl(cxn, "CREATE SEQUENCE FRIENDS_ID_SEQ");
    }
    if (!H2Utils.tableExists(cxn, "FRIENDS")) {
      H2Utils.ddlResource(cxn, SQL_TABLE_FRIENDS);
      String sq = "NEXT VALUE FOR FRIENDS_ID_SEQ";
      String sql = 
"INSERT INTO FRIENDS(FRIENDS_ID, FIRST_NAME, LAST_NAME, PHONE_NR,  REMARKS) VALUES" +
                    "("+sq+",    'Joe',      'Dalton',  '00-1111', 'Could be our next president')";
      H2Utils.ddl(cxn, sql); sql =
"INSERT INTO FRIENDS(FRIENDS_ID, FIRST_NAME, LAST_NAME, PHONE_NR,  REMARKS) VALUES" +
                   "("+sq+",     'Jack',      'Dalton',  '00-2222', '')";
      H2Utils.ddl(cxn, sql); sql =
"INSERT INTO FRIENDS(FRIENDS_ID, FIRST_NAME, LAST_NAME, PHONE_NR,  REMARKS) VALUES" +
                   "("+sq+",     'William',   'Dalton',  '00-2222', '')";
      H2Utils.ddl(cxn, sql); sql =
"INSERT INTO FRIENDS(FRIENDS_ID, FIRST_NAME, LAST_NAME, PHONE_NR,  REMARKS) VALUES" +
                   "("+sq+",     'Averell',   'Dalton',  '00-3333', 'Always hungry')";
      H2Utils.ddl(cxn, sql); sql =
"INSERT INTO FRIENDS(FRIENDS_ID, FIRST_NAME, LAST_NAME, PHONE_NR,  REMARKS) VALUES" +
                   "("+sq+",     'Lucky',     'Luke',    '00-1010', 'Shoots faster than his shadow')";
      H2Utils.ddl(cxn, sql);
    }
    if (!H2Utils.tableExists(cxn, "CDS")) {
      H2Utils.ddlResource(cxn, SQL_TABLE_CDS);
      String sql = 
"INSERT INTO CDS(CDS_ID,                    CD_XID,         NAME,       ARTIST,    LENDED_F,  LEND_DATE, FRIENDS_ID, REMARKS) VALUES" +
               "(NEXT VALUE FOR CDS_ID_SEQ, '12-23198-356', 'Revolver', 'Beatles', '0',       NULL,      NULL,       'Old CD, lots of scratches')" ;
      H2Utils.ddl(cxn, sql);
      sql = 
"INSERT INTO CDS(CDS_ID,                    CD_XID,         NAME,            ARTIST,          LENDED_F,  LEND_DATE, FRIENDS_ID, REMARKS) VALUES" +
               "(NEXT VALUE FOR CDS_ID_SEQ, '14-21198-380', 'Samurai Samba', 'Yellowjackets', '0',       NULL,      NULL,       'Great jazz-rock of the 80''s')" ;
      H2Utils.ddl(cxn, sql);
    }
    cxn.close();
  }
  
  public void run()
  {
    frame.refreshCdList();
    frame.refreshFriendList();
    //frame.pack();
    frame.setSize(800, 600);
    frame.setVisible(true);        
  }
  
  
  public static void main(String[] args)
  throws Exception
  {
    if (args.length != 1) {
      String msg = "Missing argument (path of properties file)";
      JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
      throw new IllegalArgumentException(msg);
    }
    LendingCds app = new LendingCds();
    app.init(args[0]);
    app.run();
  }
   
  /**
   * Get a friend, from database. Session must already be opened and
   * begun.
   * @param id
   * @return
   */
  public Friend getFriend(Integer id)
  {
    Friend f = null;
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    if (session == null) session = SSessionJdbc.open(ds, "getFriend");
    session.begin();
    FRIENDS rec = session.find(FRIENDS.meta, id);
    if (rec == null) return null;
    f = rec.makeFriend();
    session.commit();
    return f;    
  }
    
}
