package simpleorm.drivers;

import simpleorm.dataset.SFieldScalar;
import simpleorm.dataset.SRecordMeta;
import simpleorm.sessionjdbc.SDriver;
import simpleorm.sessionjdbc.SGenerator;

/**
 * This contains HSQL specific code. Note that this toy database does not
 * support locking, so is not safe in multi user mode even with optimistic
 * locking. See {@link SDriver#supportsLocking}.
 */

public class SDriverHSQL extends SDriver {

	protected String driverName() {
		return "HSQL Database Engine Driver";
	}

	/** HSQL has a major hole but optimisic locking papers over it. */
	public boolean supportsLocking() {
		return false;
	}

	@Override protected String columnTypeSQL(SFieldScalar field, String defalt) {
		if (defalt.equals("BYTES"))
			return "BINARY"; // Ie. just a byte array.
		else
			return defalt;
	}

	protected long generateKeySequence(SRecordMeta<?> rec, SFieldScalar keyFld) {
    	Object sequenceName = ((SGenerator)keyFld.getTheGenerator()).getName();

		String qry = "SELECT NEXT VALUE FOR " + (String) sequenceName
				+ " FROM DUAL"; // ### Dual!
		// Thanks elifarley. Note that HSQL also supports Identity.

		Object next = getSession().rawQuerySingle(qry, false);
		
		if (next == null)
			return 0;
		if (next instanceof Number)
			return ((Number) next).longValue();
		else
			return Long.parseLong(next.toString());
	}

	public boolean supportsKeySequences() {
		return true;
	}

	protected String createSequenceDDL(String name) {
		return "CREATE SEQUENCE " + name;
	}

	protected String dropSequenceDDL(String name) {
		return "DROP SEQUENCE " + name;
	}

    protected OffsetStrategy getOffsetStrategy() {
		return OffsetStrategy.JDBC;
	}

}
