package org.hmn.jrsd.test;

import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import oracle.sql.BLOB;

import org.hmn.jrsd.rt.BlobCollectVisitor;
import org.hmn.jrsd.rt.ClobCollectVisitor;
import org.hmn.jrsd.rt.JrsdRtUtils;
import org.hmn.jrsd.rt.ResultSetColVisitable;
import org.hmn.jrsd.rt.SormUtils;
import org.hmn.jrsd.test.ClobTests2.TEST_CLOB2;
import org.hmn.jrsd.test.ClobTests3.TEST_CLOB3;

import simpleorm.dataset.SFieldBlob;
import simpleorm.dataset.SFieldClob2;
import simpleorm.dataset.SFieldFlags;
import simpleorm.dataset.SFieldInteger;
import simpleorm.dataset.SFieldString;
import simpleorm.dataset.SQueryMode;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;
import simpleorm.dataset.SSelectMode;
import simpleorm.sessionjdbc.SSessionJdbc;

public class BlobTests3
extends OraJrsdTestCase
{
  private static final int MB = 1024 * 1024;
  private static final int GB = 1024 * MB;
  
  private static final String longString =
    "A super very long string but not that long in fact";
  
  public static final String tableName = 
    TEST_BLOB3.meta.getTableName();

  static final private byte[] testContents = {0,1,2,3,4,5};
  
  protected void setUp() throws Exception {
    super.setUp();
    SSessionJdbc s = beginConnection("BlobTests setUp");
    if (!SormUtils.tableExists(tableName)) {
      System.out.println("creation de la table "+tableName);
      //TEST_BLOB3.meta.
      //s.getDriver().dropTableNoError(TEST_BLOB3.meta.getTableName());
      ddl(s.getDriver().createTableSQL(TEST_BLOB3.meta));
    }   
    // empty_blob() is the function that initializes a blob in Oracle
    ddl("DELETE FROM TEST_BLOB3");
    ddl("INSERT INTO TEST_BLOB3(ID, NAME, SOMEBLOB)" +
        " VALUES (2, 'My null BLOB', null)");
    ddl("INSERT INTO TEST_BLOB3(ID, NAME, SOMEBLOB)" +
        " VALUES (3, 'My third BLOB', empty_blob())");
    endConnection();
  }
   
  /**
   * Test the {@link SormUtils#updateExtra(SSessionJdbc, SRecordInstance, String)}
   * and {@link SormUtils#selectExtra(SSessionJdbc, SRecordInstance, String, ResultSetColVisitable)}
   * methods.
   * @throws Exception
   */
  public void testUpdateSelectExtra()
  throws Exception
  {
    String allowedChars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-@_()[]=+%!:/;.,?*";
    SSessionJdbc s = SSessionJdbc.open(ds, "testSelectExtra");
    s.begin();
    TEST_BLOB3 rec = s.create(TEST_BLOB3.meta, 5);
    rec.setString(TEST_BLOB3.NAME, "The blob number 5");    
    s.commit();
    
    s.begin();
    rec = s.find(TEST_BLOB3.meta, 5);
    //make a 256K array of bytes
    byte[] b256k = new byte[256 * 1024];
    for (int i = 0; i < 256 * 1024; i++) {
      b256k[i] = (byte)(Math.random() * 256);
    }
    SormUtils.updateExtra(s, rec, "SOMEBLOB = empty_blob()");
    BlobCollectVisitor visitor = new BlobCollectVisitor();
    SormUtils.selectExtra(s, rec, "SOMEBLOB", visitor);
    Blob clb = visitor.getBlob(); 
    try {
      // The following method fails in Oracle, because the string is too big
      clb.setBytes(0, b256k); //Oracle error : Invalid arguments      
    }
    catch (Exception ex) {
      ex.printStackTrace(System.err);
    }
    //less convenient method, but works
    SormUtils.writeToBlob(b256k, clb);
    s.commit();
    
    s.begin();
    rec = s.find(TEST_BLOB3.meta, 5);
    visitor = new BlobCollectVisitor();
    SormUtils.selectExtra(s, rec, "SOMEBLOB", visitor);
    clb = visitor.getBlob(); 
    byte[] bytes = SormUtils.readFromBlob(clb);
    assertEquals(b256k[35], bytes[35]);
    s.commit();
    
    s.close();    
  }
  
  public void testRegularBlobUsage()
  throws Exception
  {
    SSessionJdbc s = SSessionJdbc.open(ds, "testRegularBlobUsage");
    
    //make a 256K array of bytes
    byte[] b256k = new byte[256 * 1024];
    for (int i = 0; i < 256 * 1024; i++) {
      b256k[i] = (byte)(Math.random() * 256);
    }
    
    s.begin();
    TEST_BLOB3 rec = s.create(TEST_BLOB3.meta, 6);
    rec.setString(TEST_BLOB3.NAME, "The test number 6");
    s.flush();
    SormUtils.updateExtra(s, rec, "SOMEBLOB = empty_blob()"); //oracle-specific
    s.commit();
    
    s.begin();
    rec = s.find(TEST_BLOB3.meta, SQueryMode.SFOR_UPDATE, 6);
    SormUtils.writeToBlob(s, rec, "SOMEBLOB", b256k);
    s.commit();
    
    s.begin();
    rec = s.find(TEST_BLOB3.meta, SQueryMode.SFOR_UPDATE, 6);
    byte[] bytes = SormUtils.readFromBlob(s, rec, "SOMEBLOB");
    s.commit();
    
    s.close();        
  }
    
  /**
   * Test the {@link SormUtils#updateExtra(SSessionJdbc, SRecordInstance, String)}
   * and {@link SormUtils#selectExtra(SSessionJdbc, SRecordInstance, String, ResultSetColVisitable)}
   * methods.
   * @throws Exception
   */
  public void testPsSetBytes()
  throws Exception
  {
    //make a 256K array of bytes
    byte[] b256k = new byte[256 * 1024];
    for (int i = 0; i < 256 * 1024; i++) b256k[i] = (byte)(Math.random() * 256);    
    SSessionJdbc s = SSessionJdbc.open(ds, "testSelectExtra");
    s.begin();
    TEST_BLOB3 rec = s.create(TEST_BLOB3.meta, 7);
    rec.setString(TEST_BLOB3.NAME, "The blob number 7");
    rec.setBytes(TEST_BLOB3.SOMEBLOB,  b256k);
    s.commit();
        
    s.begin();
    rec = s.findOrCreate(TEST_BLOB3.meta, SSelectMode.SALL, 7);        
    byte[] bytes = rec.getBytes(TEST_BLOB3.SOMEBLOB);
    assertEquals(b256k[900], bytes[900]);
    s.commit();
    
    s.close();    
  }
      
  public static class TEST_BLOB3
  extends SRecordInstance
  {
    static SRecordMeta<TEST_BLOB3> meta = 
      new SRecordMeta<TEST_BLOB3>(TEST_BLOB3.class, "TEST_BLOB3");
      
    static SFieldInteger ID = new SFieldInteger(meta, "ID", SFieldFlags.SPRIMARY_KEY);
    static SFieldString NAME = new SFieldString(meta, "NAME", 32);
    static SFieldBlob SOMEBLOB = new SFieldBlob(meta, "SOMEBLOB", 500000, SFieldFlags.SUNQUERIED);
    
    @Override
    public SRecordMeta<?> getMeta() {
      return meta;
    }
    
  }
  
}
