package org.hmn.jrsd.test;

import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import oracle.sql.BLOB;

import org.hmn.jrsd.rt.JrsdRtUtils;
import org.hmn.jrsd.rt.SormUtils;

import simpleorm.dataset.SFieldBlob;
import simpleorm.dataset.SFieldClob2;
import simpleorm.dataset.SFieldFlags;
import simpleorm.dataset.SFieldInteger;
import simpleorm.dataset.SFieldString;
import simpleorm.dataset.SQueryMode;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;
import simpleorm.dataset.SSelectMode;
import simpleorm.sessionjdbc.SSessionJdbc;

public class ClobTests2
extends OraJrsdTestCase
{
  private static final int MB = 1024 * 1024;
  private static final int GB = 1024 * MB;
  
  private static final String longString =
    "A super very long string but not that long in fact";
  
  public static final String tableName = 
    TEST_CLOB2.meta.getTableName();

  static final private byte[] testContents = {0,1,2,3,4,5};
  
  protected void setUp()
  throws Exception
  {
    super.setUp();
    SSessionJdbc s = beginConnection("ClobTests setUp");
    if (!SormUtils.tableExists(tableName)) {
      System.out.println("creation de la table "+tableName);
      //TEST_CLOB2.meta.
      //s.getDriver().dropTableNoError(TEST_CLOB2.meta.getTableName());
      ddl(s.getDriver().createTableSQL(TEST_CLOB2.meta));
    }   
    // empty_blob() is the function that initializes a blob in Oracle
    ddl("DELETE FROM TEST_CLOB2");
    ddl("INSERT INTO TEST_CLOB2(ID, NAME, SOMECLOB)" +
        " VALUES (2, 'My null CLOB', null)");
    ddl("INSERT INTO TEST_CLOB2(ID, NAME, SOMECLOB)" +
        " VALUES (3, 'My third CLOB', empty_clob())");
    endConnection();
  }
  
  public void testRegularClobCreation()
  throws Exception
  {
    SSessionJdbc s = beginConnection("testRegularClobCreation");
    Connection cxn = s.getJdbcConnection();
    String insSql = "INSERT INTO TEST_CLOB2(ID, NAME, SOMECLOB)" +
        " VALUES (4, 'My new CLOB #4', ?)";
    PreparedStatement ps = cxn.prepareStatement(insSql);
    String str = longString;    
    StringReader sr = new StringReader(str);
    try {
      ps.setCharacterStream(1, sr, str.length()); // error in Oracle      
    }
    catch (Throwable t) {
      t.printStackTrace(System.err);
      ps.setString(1, str); //more usual way to set content
    }
    ps.executeUpdate();
    endConnection();
    s.close();
  }
  
  public void testSimpleOrmClobSupport1()
  throws Exception
  {
    SSessionJdbc s = SSessionJdbc.open(ds, "testSimpleOrmClobSupport1");
    s.begin();
    TEST_CLOB2 rec = s.create(TEST_CLOB2.meta, 5);
    rec.setString(TEST_CLOB2.NAME, "The test number 5");
    rec.setString(TEST_CLOB2.SOMECLOB, longString);
    s.commit();
    s.close();
  }
  
  public void testSimpleOrmClobSupport2()
  throws Exception
  {
    SSessionJdbc s = SSessionJdbc.open(ds, "testSimpleOrmClobSupport1");
    s.begin();
    TEST_CLOB2 rec = s.create(TEST_CLOB2.meta, 6);
    rec.setString(TEST_CLOB2.NAME, "The test number 6");
    rec.setString(TEST_CLOB2.SOMECLOB, longString);
    s.commit();
    //now re-read
    s.begin();
    rec = s.findOrCreate(TEST_CLOB2.meta, SSelectMode.SALL, 6);
    String readContents = rec.getString(TEST_CLOB2.SOMECLOB);
    assertNotNull(readContents);
    assertEquals('e', readContents.charAt(5));
    s.commit();
    s.close();
  }
  
  public void testSelectExtra()
  throws Exception
  {
    
  }
  
  public void testSetAsString()
  throws Exception
  {
    SSessionJdbc s = SSessionJdbc.open(ds, "testSetAsString");
    s.begin();
    TEST_CLOB2 rec = s.create(TEST_CLOB2.meta, 7);
    rec.setString(TEST_CLOB2.NAME, "The test number 7");
    s.flush(rec);
    SormUtils.updateColumnAsString(s, rec, "SOMECLOB", longString);
    s.commit();
    //now re-read
    s.begin();
    rec = s.findOrCreate(TEST_CLOB2.meta, SSelectMode.SALL, SQueryMode.SFOR_UPDATE, 7);
    String readContents = rec.getString(TEST_CLOB2.SOMECLOB);
    assertNotNull(readContents);
    assertEquals(longString, readContents);
    s.commit();
    //change CLOB to random chars
    String randomContent = makeRandomString();
    s.begin();
    rec = s.find(TEST_CLOB2.meta, SQueryMode.SFOR_UPDATE, 7);
    SormUtils.updateColumnAsString(s, rec, "SOMECLOB", randomContent);    
    s.commit();
    //re-read and compare
    s.begin();
    rec = s.findOrCreate(TEST_CLOB2.meta, SSelectMode.SALL, SQueryMode.SFOR_UPDATE, 7);
    readContents = rec.getString(TEST_CLOB2.SOMECLOB);
    assertNotNull(readContents);
    assertEquals(randomContent, readContents);
    s.commit();
    s.close();    
  }
  
  private String makeRandomString()
  {
    char[] randomChars = new char[1000000];
    for (int i = 0; i < randomChars.length; i++) {
      char c = (char)(Math.random() * 96 + 32);
      randomChars[i] = c;
    }
    return new String(randomChars);
  }
  
  public static class TEST_CLOB2 extends SRecordInstance {
    static SRecordMeta<TEST_CLOB2> meta = 
      new SRecordMeta<TEST_CLOB2>(TEST_CLOB2.class, "TEST_CLOB2");
      
    static SFieldInteger ID = new SFieldInteger(meta, "ID", SFieldFlags.SPRIMARY_KEY);
    static SFieldString NAME = new SFieldString(meta, "NAME", 32);
    static SFieldClob2 SOMECLOB = new SFieldClob2(meta, "SOMECLOB", SFieldFlags.SUNQUERIED);
    
    @Override
    public SRecordMeta<?> getMeta() {
      return meta;
    }
    
  }
  
}
