package org.hmn.jrsd.test;

import java.io.FileInputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.hmn.jrsd.rt.SormUtils;

import simpleorm.dataset.SQuery;
import simpleorm.dataset.SSelectMode;
import simpleorm.sessionjdbc.SSessionJdbc;

import junit.framework.TestCase;
/*
 * 
 * Cree par Harry Karadimas a 19 sept. 2006
 */
/**
 * 
 * @author karadimas
 */
public class RefTests
extends OraJrsdTestCase
{
  static Logger lg = Logger.getLogger(MasterDetailTests.class);
  
  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp()
  throws Exception 
  {
    String tableName;
    super.setUp();
    beginConnection("setUp");    
    if (!SormUtils.tableExists("JT_A")) {
      System.out.println("Creating table JT_A");
      SormUtils.ddl("CREATE TABLE JT_A (" + 
          "  A_ID integer," + 
          "  PRIMARY KEY (A_ID)" + 
          ")"
          );
    }
    if (!SormUtils.tableExists("JT_B")) {
      System.out.println("Creating table JT_B");
      SormUtils.ddl("CREATE TABLE JT_B (" + 
          "  B_ID integer," + 
          "  A_ID integer," + 
          "  C_ID integer," + 
          "  PRIMARY KEY (B_ID)" + 
          ")"
          );
    }
    if (!SormUtils.tableExists("JT_C")) {
      System.out.println("Creating table JT_C");
      SormUtils.ddl("CREATE TABLE JT_C (" + 
          "  C_ID integer," + 
          "  PRIMARY KEY (C_ID)" + 
          ")"
          );
    }
    SormUtils.ddl("DELETE FROM JT_A");
    SormUtils.ddl("DELETE FROM JT_B");
    SormUtils.ddl("DELETE FROM JT_C");
    String colList = "MASTER_TABLE_ID, NAME";
    SormUtils.ddl("INSERT INTO JT_A(A_ID) VALUES (1)");
    SormUtils.ddl("INSERT INTO JT_A(A_ID) VALUES (2)");
    SormUtils.ddl("INSERT INTO JT_A(A_ID) VALUES (3)");
    SormUtils.ddl("INSERT INTO JT_A(A_ID) VALUES (4)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (11)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (12)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (13)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (14)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (21)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (22)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (41)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (42)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (43)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (44)");
    SormUtils.ddl("INSERT INTO JT_C(C_ID) VALUES (45)");
    
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (1,1,11)");
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (2,1,12)");
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (3,1,13)");
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (4,1,14)");
    
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (5,2,21)");
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (6,2,22)");
    
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (7,4,41)");
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (8,4,42)");
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (9,4,43)");
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (10,4,44)");
    SormUtils.ddl("INSERT INTO JT_B(B_ID,A_ID,C_ID) VALUES (11,4,45)");
    
    endConnection();
  }
    
  public void testABjoin()
  {
    SSessionJdbc session = beginConnection("testABjoin");
    StringBuffer sb = new StringBuffer();
    SQuery<JT_B_gen> q = new SQuery<JT_B_gen>(JT_B_gen.meta)
                 .join(JT_B_gen.ref_JT_A)
                 .eq(JT_A_gen.A_ID, 2);
    List<JT_B_gen> recs = session.query(q);
    for (JT_B_gen rec:recs) {
      sb.append(rec.getB_ID());      
    }
    assertEquals(sb.toString(), "56");    
    endConnection();
  }
  
  /**
   * Test de double jointure (relation n-m). Il est necessaire d'utiliser 
   * SQY_OPTIMISTIC sinon Oracle envoie une erreur (impossible d'utiliser a la 
   * fois DISTINCT et FOR UPDATE).   
   */
  public void testABCjoin()
  {
    SSessionJdbc session = beginConnection("testABCjoin");
    StringBuffer sb = new StringBuffer();    
//    SQuery<JT_C> q = new SQuery<JT_C>(JT_C.meta)
//                 .join(JT_B_gen.ref_JT_C)
//                 .join(JT_B_gen.ref_JT_A)
//                 .eq(JT_A_gen.cA_ID, 4);
    SQuery<JT_C> q2 = new SQuery<JT_C>(JT_C.meta)
                 .fieldRelopParameter(JT_C.C_ID, "=", JT_B_gen.C_ID)
                 .fieldRelopParameter(JT_B_gen.A_ID, "=", JT_A_gen.A_ID)
                 .fieldRelopParameter(JT_A_gen.A_ID, "=", 4);
    q2.getJoinTables().put(JT_B_gen.meta, SSelectMode.SNORMAL);
    q2.getJoinTables().put(JT_A_gen.meta, SSelectMode.SNORMAL);
    //SQuery<JT_C> q =  
    
    
    List<JT_C> recs = session.query(q2);
    for (JT_C rec:recs) {
      sb.append(rec.getC_ID());      
    }
    assertEquals(sb.toString(), "4142434445");
    //S ystem.out.println("sb="+sb);
    endConnection();
  }
  
  public void testABCjoin2()
  {
    SSessionJdbc session = beginConnection("testABCjoin");
    StringBuffer sb = new StringBuffer();    
    SQuery<JT_C> q = new SQuery<JT_C>(JT_C.meta);
    q.rawJoin(", JT_B, JT_A")
     .fieldRelopParameter(JT_C.C_ID, "=", JT_B_gen.C_ID)
     .fieldRelopParameter(JT_B_gen.A_ID, "=", JT_A_gen.A_ID)
     .fieldRelopParameter(JT_A_gen.A_ID, "=", 4);
    
    List<JT_C> recs = session.query(q);
    for (JT_C rec:recs) {
      sb.append(rec.getC_ID());      
    }
    assertEquals(sb.toString(), "4142434445");
    //S ystem.out.println("sb="+sb);
    endConnection();    
  }
}
