package org.hmn.jrsd;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import antlr.Token;

/*
 * 
 * Cree par Harry Karadimas a 31 juil. 2006
 */
/**
 * Declare a mapping with the table. Mappings are used by simple orm classes,
 * transport objects, and field sets.
 * @author karadimas
 */
public class MappingDecl
{
  String comment;
  String mappingClass;
  String mappingType;
  String alias;
  TableDef tDef;
  ArrayList<FieldDecl> fieldDecls = new ArrayList<FieldDecl>();
  String superClass;
  String extraCode;
  List<String> sormFlags = new ArrayList<String>();
  JrsdSpec jrsdSpec;
  boolean superClassAbstract;
  Boolean quoting = null;
  
  public void addFieldDecl(FieldDecl fd)
  {
    fieldDecls.add(fd);
    fd.setParent(this);
  }
  
  /**
   * 
   */
  public MappingDecl(String type, String name, String alias)
  {
    this.mappingClass = name;
    this.mappingType = type;
    this.alias = alias;
  }
  
  public List<FieldDecl> getFieldDeclarations()
  {
    return fieldDecls;
  }
  
  public List<MappingDecl> getRelatedMappings(String type)
  {    
    HashSet<MappingDecl> mps = new HashSet<MappingDecl>();
    for (Iterator<FieldDecl> iter = fieldDecls.iterator(); iter.hasNext();) {
      FieldDecl fd = iter.next();
      for (Iterator<FieldDecl> iter2 = fd.getSubfields().iterator(); iter2.hasNext();) {
        FieldDecl sfd = iter2.next();
        MappingDecl p = sfd.getParent(); 
        if (p != null && type.equals(p.getMappingType())) mps.add(p);        
      }
    }
    if (mps.isEmpty()) return null;
    return new ArrayList<MappingDecl>(mps);
  }
  
  public List<MappedFieldsSet> getMappedFieldSets(String type)
  {
    HashMap<MappingDecl, MappedFieldsSet> setsByMapping = 
      new HashMap<MappingDecl, MappedFieldsSet>();
    for (Iterator<FieldDecl> iter = fieldDecls.iterator(); iter.hasNext();) {
      FieldDecl fd = (FieldDecl) iter.next();
      for (Iterator<FieldDecl> iter2 = fd.getSubfields().iterator(); 
           iter2.hasNext();) 
      {
        FieldDecl sfd = iter2.next();
        MappingDecl p = sfd.getParent();
        MappedFieldsSet mfs = (MappedFieldsSet) setsByMapping.get(p);
        if (mfs == null) {
          mfs = new MappedFieldsSet();
          mfs.setMapping(p);
          if (type.equals(p.getMappingType())) setsByMapping.put(p, mfs);
        }
        mfs.addField(sfd);                
      }
    }
    Collection<MappedFieldsSet> values = setsByMapping.values();
    return new ArrayList<MappedFieldsSet>(values);
  }
  
  public List<MappedFieldsSet> getTransportMfss()
  {
    return getMappedFieldSets("transport");    
  }
  
  public List<MappedFieldsSet> getFieldsetMfss()
  {
    return getMappedFieldSets("fieldset");    
  }
  
  public String getAlias()
  {
    return alias;
  }
  
  public void setAlias(String alias)
  {
    this.alias = alias;
  }
  
  public String getComment()
  {
    return comment;
  }
  
  public void setComment(String comment)
  {
    this.comment = comment;
  }
  
  public void setComment(Token t) { 
    setComment(t == null ? null : t.getText()); 
  }
  
  public String getMappingClass()
  {
    return mappingClass;
  }
  
  public String getMappingClassNameOnly()
  {
    int lastDotPos = mappingClass.lastIndexOf('.');
    if (lastDotPos < 0) return mappingClass;
    return mappingClass.substring(lastDotPos+1);
  }
  
  public String getMappingPackage()
  {
    int lastDotPos = mappingClass.lastIndexOf('.');
    if (lastDotPos < 0) return null;
    return mappingClass.substring(0, lastDotPos);    
  }
  
  public void setMappingClass(String mappingClassName)
  {
    this.mappingClass = mappingClassName;
  }
  
  public String getMappingType()
  {
    return mappingType;
  }
  
  public void setMappingType(String mappingType)
  {
    this.mappingType = mappingType;
  }
  
  public TableDef getTableDef()
  {
    return tDef;
  }
  
  public void setTableDef(TableDef tableDef)
  {
    this.tDef = tableDef;
  }
  
  public void addError(JrsdError e) { tDef.addError(e); }
  
  public String getSuperClass()
  {
    return superClass;
  }
  
  public String getSuperClassNameOnly()
  {
    int lastDotPos = superClass.lastIndexOf('.');
    if (lastDotPos < 0) return superClass;
    return superClass.substring(lastDotPos+1);
  }
  
  public String getSuperPackage()
  {
    int lastDotPos = superClass.lastIndexOf('.');
    if (lastDotPos < 0) return null;
    return superClass.substring(0, lastDotPos);    
  }
  
  public void setSuperClass(String superClass)
  {
    this.superClass = superClass;
  }
  
  public void setSuperClass(Token t)
  {
    this.superClass = (t == null) ? null : t.getText();
  }
  
  public String getSormQualifiedClassName()
  {
    if (superClass != null) return superClass;
    return mappingClass;
  }
  
  public String getSormClassNameOnly()
  {
    if (superClass != null) return getSuperClassNameOnly();
    return getMappingClassNameOnly();
  }
  
  public String getSormPackage()
  {
    if (superClass != null) return getSuperPackage();
    return getMappingPackage();
  }
  
  public String getExtraCode()
  {
    if (extraCode == null) return null;
    String ec = extraCode.substring(1);
    ec = ec.substring(0, ec.length()-1);
    return ec;
  }
  
  public void setExtraCode(String extraCode)
  {
    this.extraCode = extraCode;
  }
  
  public void setExtraCode(Token t) {
    setExtraCode(t == null ? null : t.getText()); 
  }
  
  public List<String> getSormFlags()
  {
    return sormFlags;
  }
  
  public void addSormFlag(String flag)
  {
    sormFlags.add(flag);
  }
  
  public void setSormFlags(List<String> sormFlags)
  {
    this.sormFlags = sormFlags;
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return "MappingDecl=[name="+mappingClass+",type="+mappingType+"]";
  }
  
  public boolean isAccessorGenerationEnabled()
  {
    return !sormFlags.contains("noaccessors");
  }
  
  public JrsdSpec getJrsdSpec() {
    if (jrsdSpec != null) return jrsdSpec;
    if (tDef == null) return null;
    return tDef.getJrsdSpec();
  }
  
  public void setJrsdSpec(JrsdSpec jrsdSpec) {
    this.jrsdSpec = jrsdSpec;
  }

  public boolean isSuperClassAbstract() {
    return superClassAbstract;
  }

  public void setSuperClassAbstract(boolean superClassAbstract) {
    this.superClassAbstract = superClassAbstract;
  }
  
  /**
   * When the name is a class, get only the name, without the package.
   * @return
   */
  public String getUnpackagedName()
  {
    int ix = mappingClass.lastIndexOf('.');
    if (ix < 0) return mappingClass;
    return mappingClass.substring(ix + 1);
  }
  
  public boolean isQuotingNecessary()
  {
    //look if quoting was forced to off or on
    Boolean quotingForced = getTableDef().getQuotingForced(); 
    if (quotingForced != null) return quotingForced.booleanValue();
    //try to guess if quoting is needed
    String tableName = getTableDef().getName();
    return Jrsd.isQuotingNecessary(tableName);
  }
    
}
