package org.hmn.jrsd;

import java.io.StringWriter;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.antlr.stringtemplate.NoIndentWriter;
import org.antlr.stringtemplate.StringTemplate;
import org.jdom.Comment;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Emitter for xml format. Depends on jdom.
 * @author 3073951
 *
 */
public class XmlEmitter
{
  
  public XmlEmitter() { }
  
  public void produceXml(StringBuffer sb, JrsdSpec spec)
  {   
    Document doc = new Document();    
    Element root = new Element("jrsd-tables");
    doc.addContent(root);
    Comment comnt = new Comment("Sql elements declared as XML to facilitate " +
    		"translation via XSL for other\r\n tools. Generated on " + new Date());
    root.addContent(comnt);
    Iterator<TableDef> iter = spec.getTableDefs().iterator();
    while (iter.hasNext()) {
      TableDef td = (TableDef) iter.next();
      Element xtd = new Element("table-def");
      xtd.setAttribute("name", td.getName());
      xtd.setAttribute("ver", String.valueOf(td.getVersion()));
      nea(xtd, "comment", String.valueOf(td.getComment()));
      addColumns(xtd, td);
      
      if (!isEmpty(td.getExtraSql())) {
        Element xExtraSql = new Element("extra-sql");
        xExtraSql.addContent(td.getExtraSql());
        xtd.addContent(xExtraSql);
      }
      
      if (!isEmpty(td.getLiteralSql())) {
        Element xLitSql = new Element("lit-sql");
        xLitSql.addContent(td.getLiteralSql());
        xtd.addContent(xLitSql);
      }
      
      if (!isEmpty(td.getPostSql())) {
        Element xPostSql = new Element("post-sql");      
        xPostSql.addContent(td.getPostSql());
        xtd.addContent(xPostSql);
      }
      
      
      root.addContent(xtd);
    }//while
    Format xmlFormat = Format.getPrettyFormat();
    XMLOutputter xoutp = new XMLOutputter(xmlFormat);
    sb.append(xoutp.outputString(doc));
  }
  
  void addColumns(Element xtd, TableDef td)
  {
    List<ColumnDef> cdefs = td.getColDefs();
    for (ColumnDef cdef : cdefs) {
      if (cdef.isPrimaryKeyDecl()) {
        Element xpkdef = new Element("primary-key-def");
        //dump primary keys as sub-elements
        List<String> pks = cdef.getPrimaryKeys();
        if (pks != null) {
          for (String pk : pks) {
            Element xpk = new Element("pk");
            xpk.addContent(pk);
            xpkdef.addContent(xpk);
          }        
        }        
        xtd.addContent(xpkdef);
      }//if (cdef.isPrimaryKeyDecl())
      else {
        Element xcdef = new Element("column-def");
        nea(xcdef, "name", cdef.getName());
        nea(xcdef, "comment", cdef.getComment());
        nea(xcdef, "literal-sql", cdef.getLiteralSql());
        ColumnTypeDef ct = cdef.getType();
        if (ct != null) {
          Element xct = new Element("col-type");
          nea(xct, "type-name", ct.getTypeName());
          nea(xct, "comment", ct.getComment());
          nea(xct, "la", ct.getLa());
          nea(xct, "lb", ct.getLb());
          nea(xct, "lc", ct.getLc());
          nea(xct, "ld", ct.getLd());
          nea(xct, "le", ct.getLe());
          nea(xct, "lf", ct.getLf());
          nea(xct, "literal-sql", ct.getLiteralSql());
          // param1 is also used as "precision"
          nea(xct, "param1", ct.getParam1());          
          nea(xct, "scale", ct.getScale());
          nea(xct, "equiv-sql-type-name", ct.inferEquivalentSqlTypeName());
          nea(xct, "equiv-sql-type", ct.inferEquivalentSqlType());
          xcdef.addContent(xct);        
        }
        xtd.addContent(xcdef);        
      }//else of (cdef.isPrimaryKeyDecl())
    }//for
  }
  
  void nea(Element e, String attName, String attVal)
  {
    if (!isEmpty(attVal)) e.setAttribute(attName, attVal);
  }
  
  /** trivial method to test if a string is null or contains only spaces */
  public static final boolean isEmpty(String str) {
    return str == null || str.trim().equals("");
  }
  
}
