package org.hmn.jrsd.rt;

import simpleorm.dataset.SFieldScalar;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;

/**
 * Information that uniquely identifies a record : table name, primary key
 * names, primary key values.
 * @author Karadimas
 *
 */
public class RecordIdentifier
{
  private String tableName;
  private String[] pkNames;
  private Object[] pkValues;
  
  public RecordIdentifier() {
  }
  
  public RecordIdentifier(String tableName, String[] pkNames, Object[] pkVals) {
    this.tableName = tableName;
    this.pkNames = pkNames;
    this.pkValues = pkVals;
  }
  
  public static RecordIdentifier makeRecordIdentifier(SRecordInstance rec)
  {
    SRecordMeta<? extends SRecordInstance> rmeta = rec.getMeta();
    SFieldScalar[] pks = rmeta.getPrimaryKeys();
    String[] pkNames = new String[pks.length];
    Object[] pkValues = new Object[pks.length];
    for (int i = 0; i < pks.length; i++) {
      pkNames[i] = pks[i].getColumnName();
      pkValues[i] = rec.getObject(pks[i]);
    }
    String tableName = rmeta.getTableName();
    RecordIdentifier ri = new RecordIdentifier(tableName, pkNames, pkValues);
    return ri;
  }
  
  public String getTableName() {
    return tableName;
  }
  public void setTableName(String tableName) {
    this.tableName = tableName;
  }
  public String[] getPkNames() {
    return pkNames;
  }
  public void setPkNames(String[] pkNames) {
    this.pkNames = pkNames;
  }
  public Object[] getPkValues() {
    return pkValues;
  }
  public void setPkValues(Object[] pkVals) {
    this.pkValues = pkVals;
  }
}
